@@ -167,6 +167,7 @@ struct SSLSupport *newSSL(void) {
 
 void initSSL(struct SSLSupport *ssl) {
   ssl->enabled               = serverSupportsSSL();
+  ssl->force                 = 0;
   ssl->sslContext            = NULL;
   ssl->sniCertificatePattern = NULL;
   ssl->generateMissing       = 0;
@@ -410,7 +411,7 @@ static void sslGenerateCertificate(const char *certificate,
     if (!WIFEXITED(status) || WEXITSTATUS(status) != 0) {
       warn("[ssl] Failed to generate self-signed certificate \"%s\"!", certificate);
     } else {
-      info("[ssl] Certificate succesfully generated.");
+      info("[ssl] Certificate successfully generated.");
     }
   }
 }
@@ -674,7 +675,7 @@ static SSL_CTX *sslMakeContext(void) {
 
   SSL_CTX_set_info_callback(context, sslInfoCallback);
 
-  debug("[ssl] Server context succesfully initialized...");
+  debug("[ssl] Server context successfully initialized...");
   return context;
 }
 #endif
@@ -894,6 +895,12 @@ int sslEnable(struct SSLSupport *ssl, int enabled) {
   return old;
 }
 
+int sslForce(struct SSLSupport *ssl, int force) {
+  int old      = ssl->force;
+  ssl->force   = force;
+  return old;
+}
+
 void sslBlockSigPipe(void) {
   sigset_t set;
   sigemptyset(&set);
