@@ -172,6 +172,13 @@ static int (*x_misc_conv)(int, const struct pam_message **,
 static int   launcher = -1;
 static uid_t restricted;
 
+// From shellinabox/shellinaboxd.c
+extern int enableUtmpLogging;
+
+#if defined(HAVE_SECURITY_PAM_APPL_H)
+static int pamSessionSighupFlag;
+#endif
+
 // MacOS X has a somewhat unusual definition of getgrouplist() which can
 // trigger a compile warning.
 #if defined(HAVE_GETGROUPLIST_TAKES_INTS)
@@ -430,7 +437,7 @@ static void loadPAM(void) {
         *symbols[i].var = (void *)my_misc_conv;
         continue;
       }
-      debug("Failed to load PAM support. Could not find \"%s\"",
+      debug("[server] Failed to load PAM support. Could not find \"%s\"!",
             symbols[i].fn);
       for (unsigned j = 0; j < sizeof(symbols)/sizeof(symbols[0]); j++) {
         *symbols[j].var = NULL;
@@ -438,7 +445,7 @@ static void loadPAM(void) {
       return;
     }
   }
-  debug("Loaded PAM suppport");
+  debug("[server] Loaded PAM suppport");
 }
 #endif
 
@@ -566,7 +573,7 @@ int terminateChild(struct Session *session) {
   }
 
   if (session->pid < 1) {
-    debug("Child pid for termination not valid!");
+    debug("[server] Child pid for termination not valid!");
     return -1;
   }
 
@@ -576,7 +583,7 @@ int terminateChild(struct Session *session) {
   check(request        = calloc(len, 1));
   request->terminate   = session->pid;
   if (NOINTR(write(launcher, request, len)) != len) {
-    debug("Child %d termination request failed!", request->terminate);
+    debug("[server] Child %d termination request failed!", request->terminate);
     free(request);
     return -1;
   }
@@ -673,15 +680,17 @@ void destroyUtmp(struct Utmp *utmp) {
       UNUSED_RETURN(setresuid(0, 0, 0));
       UNUSED_RETURN(setresgid(0, 0, 0));
 
-      setutxent();
-      pututxline(&utmp->utmpx);
-      endutxent();
+      if (enableUtmpLogging) {
+        setutxent();
+        pututxline(&utmp->utmpx);
+        endutxent();
 
 #if defined(HAVE_UPDWTMP) || defined(HAVE_UPDWTMPX)
-      if (!utmp->useLogin) {
-        updwtmpx("/var/log/wtmp", &utmp->utmpx);
-      }
+        if (!utmp->useLogin) {
+          updwtmpx("/var/log/wtmp", &utmp->utmpx);
+        }
 #endif
+      }
 
       // Switch back to the lower privileges
       check(!setresgid(r_gid, e_gid, s_gid));
@@ -850,7 +859,7 @@ static int forkPty(int *pty, int useLogin, struct Utmp **utmp,
           ptyPath[5]        = 't';
         }
         if ((slave          = NOINTR(open(ptyPath, O_RDWR|O_NOCTTY))) >= 0) {
-          debug("Opened old-style pty: %s", ptyPath);
+          debug("[server] Opened old-style pty: %s", ptyPath);
           goto success;
         }
         NOINTR(close(*pty));
@@ -1230,7 +1239,7 @@ static pam_handle_t *internalLogin(struct Service *service, struct Utmp *utmp,
 
   // Update utmp/wtmp entries
 #ifdef HAVE_UTMPX_H
-  if (service->authUser != 2 /* SSH */) {
+  if (enableUtmpLogging && service->authUser != 2 /* SSH */) {
     memset(&utmp->utmpx.ut_user, 0, sizeof(utmp->utmpx.ut_user));
     strncat(&utmp->utmpx.ut_user[0], service->user,
             sizeof(utmp->utmpx.ut_user) - 1);
@@ -1490,6 +1499,17 @@ void setWindowSize(int pty, int width, int height) {
   }
 }
 
+#if defined(HAVE_SECURITY_PAM_APPL_H)
+static void pamSessionSighupHandler(int sig ATTR_UNUSED,
+                                    siginfo_t *info ATTR_UNUSED,
+                                    void *unused ATTR_UNUSED) {
+  UNUSED(sig);
+  UNUSED(info);
+  UNUSED(unused);
+  pamSessionSighupFlag = 1;
+}
+#endif
+
 static void childProcess(struct Service *service, int width, int height,
                          struct Utmp *utmp, const char *peerName, const char *realIP,
                          const char *url) {
@@ -1550,22 +1570,24 @@ static void childProcess(struct Service *service, int width, int height,
   UNUSED_RETURN(setresuid(0, 0, 0));
   UNUSED_RETURN(setresgid(0, 0, 0));
 #ifdef HAVE_UTMPX_H
-  setutxent();
-  struct utmpx utmpx            = utmp->utmpx;
-  if (service->useLogin || service->authUser) {
-    utmpx.ut_type               = LOGIN_PROCESS;
-    memset(utmpx.ut_host, 0, sizeof(utmpx.ut_host));
-  }
-  pututxline(&utmpx);
-  endutxent();
+  if (enableUtmpLogging) {
+    setutxent();
+    struct utmpx utmpx            = utmp->utmpx;
+    if (service->useLogin || service->authUser) {
+      utmpx.ut_type               = LOGIN_PROCESS;
+      memset(utmpx.ut_host, 0, sizeof(utmpx.ut_host));
+    }
+    pututxline(&utmpx);
+    endutxent();
 
 #if defined(HAVE_UPDWTMP) || defined(HAVE_UPDWTMPX)
-  if (!utmp->useLogin) {
-    memset(&utmpx.ut_user, 0, sizeof(utmpx.ut_user));
-    strncat(&utmpx.ut_user[0], "LOGIN", sizeof(utmpx.ut_user) - 1);
-    updwtmpx("/var/log/wtmp", &utmpx);
-  }
+    if (!utmp->useLogin) {
+      memset(&utmpx.ut_user, 0, sizeof(utmpx.ut_user));
+      strncat(&utmpx.ut_user[0], "LOGIN", sizeof(utmpx.ut_user) - 1);
+      updwtmpx("/var/log/wtmp", &utmpx);
+    }
 #endif
+  }
 #endif
 
   // Create session. We might have to fork another process as PAM wants us
@@ -1578,7 +1600,7 @@ static void childProcess(struct Service *service, int width, int height,
 #if defined(HAVE_SECURITY_PAM_APPL_H)
     if (pam && !geteuid()) {
       if (pam_open_session(pam, PAM_SILENT) != PAM_SUCCESS) {
-        fprintf(stderr, "Access denied.\n");
+        fprintf(stderr, "[server] Unable to open PAM session!\n");
         _exit(1);
       }
       pid_t pid                 = fork();
@@ -1588,10 +1610,27 @@ static void childProcess(struct Service *service, int width, int height,
       case 0:
         break;
       default:;
-        // Finish all pending PAM operations.
+        // This process is used for finishing all pending PAM operations.
+        struct sigaction sa;
+        memset(&sa, 0, sizeof(sa));
+        sa.sa_flags             = SA_NOCLDSTOP | SA_SIGINFO;
+        sa.sa_sigaction         = pamSessionSighupHandler;
+        check(!sigaction(SIGHUP, &sa, NULL));
         int status, rc;
-        check(NOINTR(waitpid(pid, &status, 0)) == pid);
-        rc = pam_close_session(pam, PAM_SILENT);
+        while (1) {
+          pamSessionSighupFlag  = 0;
+          int val               = waitpid(pid, &status, 0);
+          if (val < 0 && errno == EINTR) {
+            if (pamSessionSighupFlag) {
+              // If SIGHUP signal is received it needs to be forwarded to child
+              // process, which is acctual service process.
+              kill(pid, SIGHUP);
+            }
+          } else {
+            break;
+          }
+        }
+        rc                      = pam_close_session(pam, PAM_SILENT);
         pam_end(pam, rc | PAM_DATA_SILENT);
         _exit(WIFEXITED(status) ? WEXITSTATUS(status) : -WTERMSIG(status));
       }
@@ -1629,6 +1668,10 @@ static void childProcess(struct Service *service, int width, int height,
     }
   }
 
+  // Reset the sigaction for HUP to the default, so the child does not inherit the action of SIG_IGN from us.
+  // We need the child to be able to get HUP's because we send HUP if the session times out/closes.
+  signal(SIGHUP, SIG_DFL);
+
   // Finally, launch the child process.
   if (service->useLogin == 1) {
     // At login service launch, we try to pass real IP in '-h' parameter. Real
@@ -1672,7 +1715,7 @@ static void launcherDaemon(int fd) {
     int len                   = read(fd, &request, sizeof(request));
     if (len != sizeof(request) && errno != EINTR) {
       if (len) {
-        debug("Failed to read launch request");
+        debug("[server] Failed to read launch request!");
       }
       break;
     }
@@ -1682,7 +1725,8 @@ static void launcherDaemon(int fd) {
     int   status;
     pid_t pid;
     while (NOINTR(pid = waitpid(-1, &status, WNOHANG)) > 0) {
-      debug("Child %d exited with exit code %d", pid, WEXITSTATUS(status));
+      debug("[server] Child %d exited with exit code %d.",
+            pid, WEXITSTATUS(status));
       if (WIFEXITED(status) || WIFSIGNALED(status)) {
         char key[32];
         snprintf(&key[0], sizeof(key), "%d", pid);
@@ -1699,10 +1743,10 @@ static void launcherDaemon(int fd) {
       errno = 0;
       NOINTR(pid = waitpid(request.terminate, &status, WNOHANG));
       if (pid == 0 && errno == 0) {
-        if (kill(request.terminate, SIGTERM) == 0) {
-          debug("Terminating child %d (kill)", request.terminate);
+        if (kill(request.terminate, SIGHUP) == 0) {
+          debug("[server] Terminating child %d! [HUP]", request.terminate);
         } else {
-          debug("Terminating child %d failed [%s]", request.terminate,
+          debug("[server] Terminating child %d failed! [%s]", request.terminate,
                 strerror(errno));
         }
       }
@@ -1714,12 +1758,13 @@ static void launcherDaemon(int fd) {
   readURL:
     len                       = read(fd, url, request.urlLength + 1);
     if (len != request.urlLength + 1 && errno != EINTR) {
-      debug("Failed to read URL");
+      debug("[server] Failed to read URL!");
       free(url);
       break;
     }
     while (NOINTR(pid = waitpid(-1, &status, WNOHANG)) > 0) {
-      debug("Child %d exited with exit code %d", pid, WEXITSTATUS(status));
+      debug("[server] Child %d exited with exit code %d.", pid,
+            WEXITSTATUS(status));
       if (WIFEXITED(status) || WIFSIGNALED(status)) {
         char key[32];
         snprintf(&key[0], sizeof(key), "%d", pid);
@@ -1776,7 +1821,7 @@ static void launcherDaemon(int fd) {
           childProcesses      = newHashMap(destroyUtmpHashEntry, NULL);
         }
         addToHashMap(childProcesses, utmp->pid, (char *)utmp);
-        debug("Child %d launched", pid);
+        debug("[server] Child %d launched", pid);
       } else {
         int fds[2];
         if (!pipe(fds)) {
@@ -1830,9 +1875,9 @@ int forkLauncher(void) {
     lowerPrivileges();
     closeAllFds((int []){ pair[1], 2 }, 2);
     launcherDaemon(pair[1]);
-    fatal("exit() failed!");
+    fatal("[server] Launcher exit() failed!");
   case -1:
-    fatal("fork() failed!");
+    fatal("[server] Launcher fork() failed!");
   default:
     NOINTR(close(pair[1]));
     launcher = pair[0];
